#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
/* Attention, j'utilise la bibliothque de Kayl pour  */
/* afficher les axes donc pensez  l'inclure */
#include "sdlglutils.h"

/* CONSTANTES */

#define LARGEUR 320
#define HAUTEUR 240
#define REFRESH_DELAY 20

#define MAX_PARTICLES 1000

typedef struct		   // Structure d'une particule
{   bool  active;          // Active (1=Oui/0=Non)
    double life;	   // Dure de vie
    double fade;	   // Vitesse de disparition
    double   r, g, b;	   // Valeurs RGB pour la couleur
    double x, y, z;	   // Position
    double xi, yi, zi;	   // Vecteur de dplacement
    double xg, yg, zg;	   // Gravit
}particles;

/* Tableau de stockage des particules */
particles particle[MAX_PARTICLES];

/* FONCTIONS */
double myRand(double min, double max);
int dessinerScene();
int initParticles();
int dessinerParticules();

/* FONCTION PRINCIPALE */
int main(int argc, char *argv[])
{   SDL_Surface *ecran = NULL;
    SDL_Event event;
    int continuer = 1;
    int previousTime = SDL_GetTicks();
    int elapsedTime = 0;

    /* INITIALISATIONS */
    SDL_Init(SDL_INIT_VIDEO); /* Initialisation de la SDL */
    SDL_WM_SetCaption("Particle Engine", NULL);
    ecran = SDL_SetVideoMode(LARGEUR, HAUTEUR, 32, SDL_OPENGL); /* Ouverture de la fentre */

    /* Initialisation du mode 3D */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );
    gluPerspective(70,(double)LARGEUR/HAUTEUR,1,1000);

    glEnable(GL_DEPTH_TEST);  /* Initialisation du Z-Buffer */
    glEnable(GL_BLEND);			// Autoriser la transparence
    glBlendFunc(GL_SRC_ALPHA,GL_ONE);	// Type de transparence
    
    /* Ces 2 lignes amliorent le rendu mais ne sont pas ncessaires */
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	                glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // Efface le tampon d'affichage
    glFlush();
    SDL_GL_SwapBuffers();

    initParticles();
    
    /* Boucle Gnrale */
    while (continuer)
    {   SDL_PollEvent(&event);
        switch(event.type)
        {   case SDL_QUIT:
                continuer = 0;
                break;
            case SDL_KEYDOWN:
                 switch(event.key.keysym.sym)
                 {   case SDLK_ESCAPE:
                          continuer=0;
                          break;
                     default:break;
                 }
                 break;
        }

        /* Appel de la fonction principale de dessin */
        dessinerScene();

        /* On attend le tour suivant */
        elapsedTime = SDL_GetTicks() - previousTime;
        if (elapsedTime < REFRESH_DELAY)
        {
            SDL_Delay(REFRESH_DELAY - elapsedTime);
        }
        previousTime = SDL_GetTicks();

    }

    SDL_Quit(); /* Arrt de la SDL */

    return EXIT_SUCCESS; /* Fermeture du programme */
}

/* FONCTIONS SECONDAIRES */
double myRand(double min, double max)
{   return (double) (min + ((float) rand() / RAND_MAX * (max - min + 1.0)));
}

int dessinerScene()
{   /* Initialisation */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    /* Position Camera gluLookAt(positionCamera, positionCible, vecteurVertical) */
    gluLookAt(2,2,1, 0,0,0, 0,0,1);
    drawAxis(1.0);

    /* Dessin */
    dessinerParticules();

    /* MAJ de l'cran */
    glFlush();
    SDL_GL_SwapBuffers();
    
    return 0;
}

int initParticles()
{   for(int i=0; i<MAX_PARTICLES; i++)   // Boucle sur toutes les particules
    {   particle[i].active = true;	 // On rend la particule active
	particle[i].life = 1.0;		 // Maximum de vie

        particle[i].fade = myRand(0.01,0.05);   // Vitesse de disparition alatoire

        particle[i].r=myRand(0.0,1.0);	// Quantit alatoire de Rouge
	particle[i].g=myRand(0.0,1.0);	// Quantit alatoire de Vert
	particle[i].b=myRand(0.0,1.0);	// Quantit alatoire de Bleu

        particle[i].xi = myRand(-10.0,10.0);   // Vitesse alatoire
	particle[i].yi = myRand(-10.0,10.0);
	particle[i].zi = myRand(10.0,20.0);

        particle[i].xg = 0.0;       // Gravit dirige selon l'axe -Z
	particle[i].yg = 0.0;
	particle[i].zg = -1.0;
    }
    return 0;    // Initialisation OK
}

int dessinerParticules()
{   for(int i=0; i<MAX_PARTICLES; i++) // Pour chaque particule
    {   if(particle[i].active)         // Si elle est active
        {   float x = particle[i].x;   // On rcupre sa position
            float y = particle[i].y;
	    float z = particle[i].z;

            /* Couleur de la particule; transparence = vie */
            glColor4d(particle[i].r, particle[i].g, particle[i].b, particle[i].life);

            /* Dessin de carrs  partir de deux triangles (plus rapide pour la carte video */
            glBegin(GL_TRIANGLE_STRIP);
              glVertex3d(x+0.1,y,z+0.1); // Nord-Ouest
	      glVertex3d(x-0.1,y,z+0.1); // Nord-Est
	      glVertex3d(x+0.1,y,z-0.1); // Sud-Ouest
	      glVertex3d(x-0.1,y,z-0.1); // Sud-Est
            glEnd();

            /* Dplacement */
            particle[i].x += particle[i].xi/1000;
            particle[i].y += particle[i].yi/1000;
            particle[i].z += particle[i].zi/1000;

            /* Gravit */
            particle[i].xi += particle[i].xg;
            particle[i].yi += particle[i].yg;
            particle[i].zi += particle[i].zg;

            /* "Vie" */
            particle[i].life -= particle[i].fade;

            /* Si la particule est "morte" on la rgnre */
            if (particle[i].life < 0.0)
	    {   particle[i].life = 1.0;	   // Maximum de vie
		particle[i].fade = myRand(0.01,0.05);

		particle[i].x = 0.0; // On renvoie la particule au centre
		particle[i].y = 0.0;
		particle[i].z = 0.0;

                particle[i].xi = myRand(-10.0,10.0);   // Vitesse alatoire
                particle[i].yi = myRand(-10.0,10.0);
	        particle[i].zi = myRand(10.0,20.0);

                particle[i].r=myRand(0.0,1.0);	// Quantit alatoire de Rouge
	        particle[i].g=myRand(0.0,1.0);	// Quantit alatoire de Vert
	        particle[i].b=myRand(0.0,1.0);	// Quantit alatoire de Bleu
            }
        }
    }
    return 0; // Dessin OK
}

